@@ -1,3 +1,4 @@
+import gc
 import time
 import socket
 import pickle
@@ -105,8 +106,9 @@ class CuiApp:
             if proc():
                 self._idleprocs.append(proc)
 
-        if self._idleprocs:
             return True
+        else:
+            return False
 
     def translate_theme(self, theme):
         overlays = {}
@@ -233,6 +235,10 @@ class CuiApp:
             self._input_readers.remove(reader)
 
     def run(self):
+#        def f(t, i):
+#            _trace(t, i)
+#        gc.callbacks.append(f)
+        gc.set_threshold(2000, 10, 10)
 
         nonblocking = True
         while not self._quit:
@@ -258,6 +264,8 @@ class CuiApp:
                 except InterruptedError:
                     pass
 
+                l = time.time()
+
                 if not nonblocking and not rlist:
                     # timeout
                     self._run_scheduled_task()
@@ -281,14 +289,14 @@ class CuiApp:
 
                         if self.focus.editmode:
                             self.focus.editmode.on_keyevent(self.focus, c)
-
+                            self.focus.update_window()
+                
                 if not inputs:
                     if self.mainframe.on_idle():
                         continue
+
                     # no input
                     if not self.on_idle():
-                        if self.mainframe.on_idle():
-                            continue
                         # No more idle jobs
                         nonblocking = False
                 else:
