@@ -1,3 +1,4 @@
+import re
 import itertools
 import unicodedata
 import collections
@@ -48,6 +49,10 @@ SearchOption.LAST_SEARCH = SearchOption()
 
 
 class ModeBase:
+    (UNDO_INSERT,
+     UNDO_REPLACE,
+     UNDO_DELETE) = range(3)
+
     CLOSE_ON_DEL_WINDOW = True
 
     SCREEN_NOWRAP = False
@@ -69,7 +74,8 @@ class ModeBase:
     theme = None
     highlight = None
     _check_fileupdate = 0
-
+    _last_autoindent = None
+    
     def __init__(self):
         self.commands = {}
         self.is_availables = {}
@@ -91,6 +97,7 @@ class ModeBase:
         self.tokenizers = []
         self.init_tokenizers()
         self.stylemap = {}
+        self.stylenamemap = {}
         self.highlight = highlight.Highlighter(self.tokenizers)
 
         self.setup()
@@ -125,8 +132,11 @@ class ModeBase:
                 style = self.theme.get_style(stylename)
 
                 self.stylemap[tokenid] = style
+                self.stylenamemap[style.name] = tokenid
             else:
-                self.stylemap[tokenid] = self.theme.get_style('default')
+                style = self.theme.get_style('default')
+                self.stylemap[tokenid] = style
+                self.stylenamemap[style.name] = tokenid
 
     def on_set_document(self, document):
         self.document = document
@@ -136,6 +146,7 @@ class ModeBase:
         self.document.use_undo(self.USE_UNDO)
 
     def on_document_updated(self, pos, inslen, dellen):
+        self._last_autoindent = None
         if self.highlight:
             self.highlight.updated(self.document, pos, inslen, dellen)
 
@@ -203,16 +214,19 @@ class ModeBase:
         if stylename == 'default':
             return 0
 
-        for styleid, style in self.stylemap.items():
-            if style.name == stylename:
-                return styleid
+        styleid = self.stylenamemap.get(stylename, None)
+        if styleid is not None:
+            return styleid
+            
+        if not self.stylemap:
+            ret = 1
         else:
-            if not self.stylemap:
-                ret = 1
-            else:
-                ret = max(self.stylemap.keys()) + 1
-            self.stylemap[ret] = self.theme.get_style(stylename)
-            return ret
+            ret = max(self.stylemap.keys()) + 1
+            
+        style = self.theme.get_style(stylename)
+        self.stylemap[ret] = style
+        self.stylenamemap[stylename] = ret
+        return ret
 
     def select_theme(self, theme_name, themes):
         theme = themes.get(theme_name, None)
@@ -248,7 +262,7 @@ class ModeBase:
         pass
 
     def on_str(self, wnd, s):
-        self.edit_commands.put_string(wnd, s)
+        self.put_string(wnd, s)
         wnd.screen.selection.clear()
 
         if kaa.app.macro.is_recording():
@@ -296,6 +310,102 @@ class ModeBase:
     def on_cursor_located(self, wnd, pos, y, x):
         pass
 
+    def insert_string(self, wnd, pos, s, update_cursor=True):
+        """Insert string"""
+
+        cur_pos = wnd.cursor.pos
+
+        wnd.document.insert(pos, s)
+
+        if update_cursor:
+            wnd.cursor.setpos(wnd.cursor.pos + len(s))
+            wnd.cursor.savecol()
+
+        if wnd.document.undo:
+            wnd.document.undo.add(self.UNDO_INSERT, pos, s,
+                                  cur_pos, wnd.cursor.pos)
+
+        self.on_edited(wnd)
+
+    def replace_string(self, wnd, pos, posto, s, update_cursor=True):
+        """Replace string"""
+
+        cur_pos = wnd.cursor.pos
+
+        deled = wnd.document.gettext(pos, posto)
+        wnd.document.replace(pos, posto, s)
+
+        if update_cursor:
+            wnd.cursor.setpos(pos + len(s))
+            wnd.cursor.savecol()
+
+        if wnd.document.undo:
+            wnd.document.undo.add(self.UNDO_REPLACE, pos, posto, s,
+                                  deled, cur_pos, wnd.cursor.pos)
+
+        self.on_edited(wnd)
+
+    def delete_string(self, wnd, pos, posto, update_cursor=True):
+        """Delete string"""
+
+        cur_pos = wnd.cursor.pos
+
+        if pos < posto:
+            deled = wnd.document.gettext(pos, posto)
+            wnd.document.delete(pos, posto)
+
+            if update_cursor:
+                wnd.cursor.setpos(pos)
+                wnd.cursor.savecol()
+
+            if wnd.document.undo:
+                wnd.document.undo.add(self.UNDO_DELETE, pos, posto, deled,
+                                      cur_pos, wnd.cursor.pos)
+            self.on_edited(wnd)
+
+
+    def replace_rect(self, wnd, repto):
+        if wnd.document.undo:
+            wnd.document.undo.beginblock()
+        try:
+            (posfrom, posto, colfrom, colto
+             ) = wnd.screen.selection.get_rect_range()
+
+            for s in repto:
+                if posto <= posfrom:
+                    break
+
+                sel = wnd.screen.selection.get_col_string(
+                    posfrom, colfrom, colto)
+                if sel:
+                    f, t, org = sel
+                    if org.endswith('\n'):
+                        t = max(f, t - 1)
+                    self.replace_string(wnd, f, t, s)
+                    posto += (len(s) - (t - f))
+                posfrom = wnd.document.geteol(posfrom)
+        finally:
+            if wnd.document.undo:
+                wnd.document.undo.endblock()
+
+    def put_string(self, wnd, s):
+        s = wnd.document.mode.filter_string(wnd, s)
+
+        if wnd.screen.selection.is_selected():
+            if wnd.screen.selection.is_rectangular():
+                if '\n' not in s:
+                    self.replace_rect(wnd, itertools.repeat(s))
+                else:
+                    self.replace_rect(wnd, s.split('\n'))
+
+            else:
+                sel = wnd.screen.selection.get_selrange()
+                f, t = sel
+                self.replace_string(wnd, f, t, s)
+        else:
+            self.insert_string(wnd, wnd.cursor.pos, s)
+
+
     def update_charattr(self, wnd):
         if wnd.charattrs:
             wnd.charattrs.clear()
@@ -441,20 +551,34 @@ class ModeBase:
         else:
             return ' ' * col
 
+    def cancel_auto_indent(self, wnd):
+        if self._last_autoindent:
+            f, t = (min(max(0, p), self.document.endpos())
+                      for p in self._last_autoindent)
+            self._last_autoindent = None
+            if f != t and wnd.cursor.pos == t:
+                s = self.document.gettext(f, t)
+                m = re.match(self.RE_WHITESPACE, s)
+                if m and m.group() == s:
+                    self.delete_string(
+                        wnd, f, t, update_cursor=False)
+                    wnd.cursor.setpos(f)
+                    
     def on_auto_indent(self, wnd):
         pos = wnd.cursor.pos
         f, t = self.get_indent_range(pos)
         indent = self.document.gettext(f, min(pos, t))
         if pos <= t:
-            self.edit_commands.insert_string(wnd, f, '\n',
-                                             update_cursor=False)
+            self.insert_string(wnd, f, '\n', update_cursor=False)
             wnd.cursor.setpos(wnd.cursor.pos + 1)
+            wnd.cursor.savecol()
         else:
             indent = '\n' + indent
-            self.edit_commands.insert_string(wnd, pos, indent,
-                                             update_cursor=False)
+            self.insert_string(wnd, pos, indent, update_cursor=False)
             wnd.cursor.setpos(pos + len(indent))
-
+            wnd.cursor.savecol()
+            self._last_autoindent = (pos+1, wnd.cursor.pos)
+            
         wnd.cursor.savecol()
 
     def calc_cols(self, f, t):
