@@ -85,12 +85,13 @@ class SelectItemList(dialogmode.DialogMode):
             style = item.style
 
         f, t = self.document.marks[item]
-        self.document.styles.setints(f, t, self.get_styleid(style))
+        self.document.setstyles(f, t, self.get_styleid(style))
         if activate:
             wnd.screen.apply_updates()
             top = not middle and not bottom
             wnd.screen.locate(f, top=top, middle=middle, bottom=bottom)
-
+            wnd.update_window()
+            
     def update_sel(self, wnd, newsel, middle=None, bottom=None):
         if self.cursel is not None:
             self._update_item_style(wnd, self.cursel, False)
