@@ -6,7 +6,9 @@ from kaa import keyboard
 import kaa.options
 from kaa.exceptions import KaaError
 
+
 class TestKey:
+
     def test_keytrans(self):
         cuiapp = app.CuiApp(kaa.options.build_parser().parse_args())
         assert 'A' == cuiapp.translate_key((), 'A')
@@ -20,7 +22,7 @@ class TestKey:
             cuiapp.translate_key({keyboard.shift}, 'A')
 
         assert [curses.KEY_SRIGHT] == cuiapp.translate_key(
-                {keyboard.shift}, keyboard.right)
+            {keyboard.shift}, keyboard.right)
 
     def test_ctrl(self):
         cuiapp = app.CuiApp(kaa.options.build_parser().parse_args())
@@ -31,12 +33,14 @@ class TestKey:
             cuiapp.translate_key({keyboard.ctrl}, '-')
 
         assert [curses.KEY_SRIGHT] == cuiapp.translate_key(
-                {keyboard.shift}, keyboard.right)
+            {keyboard.shift}, keyboard.right)
 
     def test_alt(self):
         cuiapp = app.CuiApp(kaa.options.build_parser().parse_args())
         assert '\x1ba' == cuiapp.translate_key({keyboard.alt}, 'a')
-        assert '\x1b\x01' == cuiapp.translate_key({keyboard.ctrl, keyboard.alt}, 'a')
-        assert ['\x1b', curses.KEY_RIGHT] == cuiapp.translate_key({keyboard.alt}, keyboard.right)
+        assert '\x1b\x01' == cuiapp.translate_key(
+            {keyboard.ctrl, keyboard.alt}, 'a')
+        assert ['\x1b', curses.KEY_RIGHT] == cuiapp.translate_key(
+            {keyboard.alt}, keyboard.right)
         assert ['\x1b', curses.KEY_SRIGHT] == cuiapp.translate_key(
-                {keyboard.alt, keyboard.shift}, keyboard.right)
+            {keyboard.alt, keyboard.shift}, keyboard.right)
